@@ -139,8 +139,7 @@ newbitarrayobject(PyTypeObject *type, Py_ssize_t nbits, int endian)
         obj->ob_item = (char *) PyMem_Malloc((size_t) nbytes);
         if (obj->ob_item == NULL) {
             PyObject_Del(obj);
-            PyErr_NoMemory();
-            return NULL;
+            return PyErr_NoMemory();
         }
     }
     obj->allocated = nbytes;
@@ -588,16 +587,10 @@ unpack(bitarrayobject *self, char zero, char one, const char *fmt)
     Py_ssize_t i;
     char *str;
 
-    if (self->nbits > PY_SSIZE_T_MAX / 8) {
-        PyErr_SetString(PyExc_OverflowError, "bitarray too large to unpack");
-        return NULL;
-    }
-
     str = (char *) PyMem_Malloc((size_t) self->nbits);
-    if (str == NULL) {
-        PyErr_NoMemory();
-        return NULL;
-    }
+    if (str == NULL)
+        return PyErr_NoMemory();
+
     for (i = 0; i < self->nbits; i++)
         str[i] = GETBIT(self, i) ? one : zero;
 
@@ -745,10 +738,9 @@ bitarray_index(bitarrayobject *self, PyObject *args)
     normalize_index(self->nbits, &stop);
 
     i = findfirst(self, vi, start, stop);
-    if (i < 0) {
-        PyErr_Format(PyExc_ValueError, "%d is not in bitarray", vi);
-        return NULL;
-    }
+    if (i < 0)
+        return PyErr_Format(PyExc_ValueError, "%d is not in bitarray", vi);
+
     return PyLong_FromSsize_t(i);
 }
 
@@ -768,11 +760,11 @@ bitarray_extend(bitarrayobject *self, PyObject *obj)
 }
 
 PyDoc_STRVAR(extend_doc,
-"extend(iterable or string, /)\n\
+"extend(iterable, /)\n\
 \n\
 Extend bitarray by appending the truth value of each element given\n\
 by iterable.  If a string is provided, each `0` and `1` are appended\n\
-as bits.");
+as bits (whitespace is ignored).");
 
 
 static PyObject *
@@ -953,11 +945,10 @@ unpickle(PyTypeObject *type, PyObject *bytes, int endian)
     data = PyBytes_AS_STRING(bytes);
     head = *data;
 
-    if (nbytes == 1 && head % 8) {
-        PyErr_Format(PyExc_ValueError,
-                     "invalid header byte 0x%02x", head);
-        return NULL;
-    }
+    if (nbytes == 1 && head % 8)
+        return PyErr_Format(PyExc_ValueError,
+                            "invalid header byte 0x%02x", head);
+
     res = newbitarrayobject(type,
                             BITS(nbytes - 1) - ((Py_ssize_t) (head % 8)),
                             endian);
@@ -1391,17 +1382,16 @@ bitarray_repr(bitarrayobject *self)
         return Py_BuildValue("s", "bitarray()");
 
     strsize = self->nbits + 12;  /* 12 is the length of "bitarray('')" */
-    if (strsize > PY_SSIZE_T_MAX / 8) {
+    if (strsize > PY_SSIZE_T_MAX) {
         PyErr_SetString(PyExc_OverflowError,
                         "bitarray too large to represent");
         return NULL;
     }
 
     str = (char *) PyMem_Malloc(strsize);
-    if (str == NULL) {
-        PyErr_NoMemory();
-        return NULL;
-    }
+    if (str == NULL)
+        return PyErr_NoMemory();
+
     /* add "bitarray('......')" to str */
     strcpy(str, "bitarray('"); /* has length 10 */
     /* don't use strcpy here, as this would add an extra null byte */
@@ -1486,10 +1476,9 @@ bitarray_remove(bitarrayobject *self, PyObject *v)
         return NULL;
 
     i = findfirst(self, vi, 0, self->nbits);
-    if (i < 0) {
-        PyErr_Format(PyExc_ValueError, "%d not in bitarray", vi);
-        return NULL;
-    }
+    if (i < 0)
+        return PyErr_Format(PyExc_ValueError, "%d not in bitarray", vi);
+
     if (delete_n(self, i, 1) < 0)
         return NULL;
     Py_RETURN_NONE;
@@ -1683,10 +1672,9 @@ bitarray_subscr(bitarrayobject *self, PyObject *item)
         return res;
     }
 
-    PyErr_Format(PyExc_TypeError,
-                 "bitarray indices must be integers or slices, not %s",
-                 Py_TYPE(item)->tp_name);
-    return NULL;
+    return PyErr_Format(PyExc_TypeError,
+                        "bitarray indices must be integers or slices, not %s",
+                        Py_TYPE(item)->tp_name);
 }
 
 /* The following functions (setslice_bitarray, setslice_bool and delslice)
@@ -2050,11 +2038,10 @@ bitarray_encode(bitarrayobject *self, PyObject *args)
         return NULL;
 
     iter = PyObject_GetIter(iterable);
-    if (iter == NULL) {
-        PyErr_Format(PyExc_TypeError, "'%s' object is not iterable",
-                     Py_TYPE(iterable)->tp_name);
-        return NULL;
-    }
+    if (iter == NULL)
+        return PyErr_Format(PyExc_TypeError, "'%s' object is not iterable",
+                            Py_TYPE(iterable)->tp_name);
+
     /* extend self with the bitarrays from codedict */
     while ((symbol = PyIter_Next(iter))) {
         value = PyDict_GetItem(codedict, symbol);
@@ -2813,7 +2800,7 @@ endian_from_string(const char* string)
     return -1;
 }
 
-static PyObject*
+static PyObject *
 bitarray_from_index(PyTypeObject *type, PyObject *index, int endian)
 {
     Py_ssize_t nbits;
@@ -3273,11 +3260,12 @@ tuple(sizeof(void *),\n\
 
 
 static PyMethodDef module_functions[] = {
-    {"get_default_endian", (PyCFunction) get_default_endian, METH_NOARGS,
-                                                   get_default_endian_doc},
-    {"_set_default_endian", (PyCFunction) set_default_endian, METH_VARARGS,
-                                                   set_default_endian_doc},
-    {"_sysinfo",   (PyCFunction) sysinfo,    METH_NOARGS,  sysinfo_doc   },
+    {"get_default_endian",  (PyCFunction) get_default_endian,
+                                      METH_NOARGS,  get_default_endian_doc},
+    {"_set_default_endian", (PyCFunction) set_default_endian,
+                                      METH_VARARGS, set_default_endian_doc},
+    {"_sysinfo",            (PyCFunction) sysinfo,
+                                      METH_NOARGS,  sysinfo_doc           },
     {NULL,         NULL}  /* sentinel */
 };
 
