@@ -16,13 +16,13 @@ from bitarray import (bitarray, frozenbitarray, bits2bytes, decodetree,
                       get_default_endian, _set_default_endian)
 from bitarray.test_bitarray import Util
 
-from bitarray.util import (zeros, urandom, pprint, make_endian, rindex, strip,
-                           count_n, count_and, count_or, count_xor, subset,
-                           ba2hex, hex2ba, ba2int, int2ba,
-                           serialize, deserialize, huffman_code)
+from bitarray.util import (
+    zeros, urandom, pprint, make_endian, rindex, strip,
+    count_n, count_and, count_or, count_xor, subset, parity,
+    ba2hex, hex2ba, ba2int, int2ba, serialize, deserialize, huffman_code
+)
 
 if sys.version_info[0] == 3:
-    unicode = str
     from io import StringIO
 else:
     from io import BytesIO as StringIO
@@ -157,7 +157,8 @@ class TestsPPrint(unittest.TestCase):
         try:
             with open(tmpfile, 'w') as fo:
                 pprint(a, fo)
-            b = eval(open(tmpfile).read())
+            with open(tmpfile, 'r') as fi:
+                b = eval(fi.read())
             self.assertEqual(a, b)
         finally:
             shutil.rmtree(tmpdir)
@@ -524,6 +525,49 @@ tests.append(TestsSubset)
 
 # ---------------------------------------------------------------------------
 
+class TestsParity(unittest.TestCase, Util):
+
+    def test_bitarray(self):
+        a = bitarray()
+        self.assertTrue(parity(a) is False)
+        par = False
+        for _ in range(100):
+            self.assertTrue(parity(a) is par)
+            a.append(1)
+            par = not par
+
+    def test_unused_bits(self):
+        a = bitarray(1)
+        a.setall(1)
+        self.assertTrue(parity(a))
+
+    def test_return_type(self):
+        p = parity(bitarray('101'))
+        self.assertIsInstance(p, bool)
+
+    def test_frozenbitarray(self):
+        self.assertTrue(parity(frozenbitarray()) is False)
+        self.assertTrue(parity(frozenbitarray('0010011')) is True)
+        self.assertTrue(parity(frozenbitarray('10100110')) is False)
+
+    def test_wrong_args(self):
+        self.assertRaises(TypeError, parity, '')
+        self.assertRaises(TypeError, bitarray(), 1)
+
+    def test_byte(self):
+        for i in range(256):
+            a = bitarray()
+            a.frombytes(bytes(bytearray([i])))
+            self.assertEqual(parity(a), a.count() % 2)
+
+    def test_random(self):
+        for a in self.randombitarrays():
+            self.assertEqual(parity(a), a.count() % 2)
+
+tests.append(TestsParity)
+
+# ---------------------------------------------------------------------------
+
 CODEDICT = {'little': {}, 'big': {
     '0': bitarray('0000'),    '1': bitarray('0001'),
     '2': bitarray('0010'),    '3': bitarray('0011'),
@@ -540,31 +584,15 @@ for k, v in CODEDICT['big'].items():
 
 class TestsHexlify(unittest.TestCase, Util):
 
-    def test_swap_hilo_bytes(self):
-        from bitarray._util import _swap_hilo_bytes
-
-        self.assertEqual(len(_swap_hilo_bytes), 256)
-        for i in range(256):
-            byte = bytes(bytearray([i]))
-            a = bitarray()
-            a.frombytes(byte)
-            self.assertEqual(len(a), 8)
-
-            b = a[4:8] + a[0:4]
-            self.assertEqual(b.tobytes(),
-                             byte.translate(_swap_hilo_bytes))
-            # with just _swap_hilo_bytes[i] we'd get an integer on Py3
-            self.assertEqual(b.tobytes(), _swap_hilo_bytes[i:i + 1])
-
     def test_ba2hex(self):
         self.assertEqual(ba2hex(bitarray(0, 'big')), '')
         self.assertEqual(ba2hex(bitarray('1110', 'big')), 'e')
         self.assertEqual(ba2hex(bitarray('1110', 'little')), '7')
-        self.assertEqual(ba2hex(bitarray('00000001', 'big')), '01')
-        self.assertEqual(ba2hex(bitarray('10000000', 'big')), '80')
-        self.assertEqual(ba2hex(bitarray('00000001', 'little')), '08')
-        self.assertEqual(ba2hex(bitarray('10000000', 'little')), '10')
-        self.assertEqual(ba2hex(frozenbitarray('11000111', 'big')), 'c7')
+        self.assertEqual(ba2hex(bitarray('0000 0001', 'big')), '01')
+        self.assertEqual(ba2hex(bitarray('1000 0000', 'big')), '80')
+        self.assertEqual(ba2hex(bitarray('0000 0001', 'little')), '08')
+        self.assertEqual(ba2hex(bitarray('1000 0000', 'little')), '10')
+        self.assertEqual(ba2hex(frozenbitarray('1100 0111', 'big')), 'c7')
         # length not multiple of 4
         self.assertRaises(ValueError, ba2hex, bitarray('10'))
         self.assertRaises(TypeError, ba2hex, '101')
@@ -582,16 +610,30 @@ class TestsHexlify(unittest.TestCase, Util):
     def test_hex2ba(self):
         _set_default_endian('big')
         self.assertEqual(hex2ba(''), bitarray())
-        for c in 'e', 'E', b'e', b'E', unicode('e'), unicode('E'):
+        for c in 'e', 'E', b'e', b'E', u'e', u'E':
             a = hex2ba(c)
             self.assertEqual(a.to01(), '1110')
             self.assertEqual(a.endian(), 'big')
-        self.assertEQUAL(hex2ba('01'), bitarray('00000001', 'big'))
+        self.assertEQUAL(hex2ba('01'), bitarray('0000 0001', 'big'))
         self.assertEQUAL(hex2ba('08', 'little'),
-                         bitarray('00000001', 'little'))
-        self.assertRaises(Exception, hex2ba, '01a7x89')
+                         bitarray('0000 0001', 'little'))
+        self.assertEQUAL(hex2ba('aD'), bitarray('1010 1101', 'big'))
+        self.assertEQUAL(hex2ba(b'10aF'),
+                         bitarray('0001 0000 1010 1111', 'big'))
+        self.assertEQUAL(hex2ba(b'10aF', 'little'),
+                         bitarray('1000 0000 0101 1111', 'little'))
+
+    def test_hex2ba_errors(self):
         self.assertRaises(TypeError, hex2ba, 0)
 
+        for endian in 'little', 'big':
+            _set_default_endian(endian)
+            self.assertRaises(ValueError, hex2ba, '01a7g89')
+            self.assertRaises(UnicodeEncodeError, hex2ba, u'10\u20ac')
+            # check for NUL bytes
+            for b in b'\0', b'\0f', b'f\0', b'\0ff', b'f\0f', b'ff\0':
+                self.assertRaises(ValueError, hex2ba, b)
+
     @staticmethod
     def hex2ba(s, endian=None):
         a = bitarray(0, endian or get_default_endian())
@@ -603,20 +645,13 @@ class TestsHexlify(unittest.TestCase, Util):
         return ''.join(a.iterdecode(CODEDICT[a.endian()]))
 
     def test_explicit(self):
-        data = [ #     little  big                  little  big
-            ('',       '',     ''),
-            ('0000',   '0',    '0'),     ('0001',   '8',    '1'),
-            ('1000',   '1',    '8'),     ('1001',   '9',    '9'),
-            ('0100',   '2',    '4'),     ('0101',   'a',    '5'),
-            ('1100',   '3',    'c'),     ('1101',   'b',    'd'),
-            ('0010',   '4',    '2'),     ('0011',   'c',    '3'),
-            ('1010',   '5',    'a'),     ('1011',   'd',    'b'),
-            ('0110',   '6',    '6'),     ('0111',   'e',    '7'),
-            ('1110',   '7',    'e'),     ('1111',   'f',    'f'),
-            ('10001100',             '13',    '8c'),
-            ('100011001110',         '137',   '8ce'),
-            ('1000110011101111',     '137f',  '8cef'),
-            ('10001100111011110100', '137f2', '8cef4'),
+        data = [ #                       little   big
+            ('',                         '',      ''),
+            ('1000',                     '1',     '8'),
+            ('1000 1100',                '13',    '8c'),
+            ('1000 1100 1110',           '137',   '8ce'),
+            ('1000 1100 1110 1111' ,     '137f',  '8cef'),
+            ('1000 1100 1110 1111 0100', '137f2', '8cef4'),
         ]
         for bs, hex_le, hex_be in data:
             a_be = bitarray(bs, 'big')
@@ -831,7 +866,7 @@ tests.append(TestsIntegerization)
 class TestsSerialization(unittest.TestCase, Util):
 
     def test_explicit(self):
-        for a, s in [
+        for a, b in [
                 (bitarray(0, 'little'),   b'\x00'),
                 (bitarray(0, 'big'),      b'\x10'),
                 (bitarray('1', 'little'), b'\x07\x01'),
@@ -839,8 +874,8 @@ class TestsSerialization(unittest.TestCase, Util):
                 (bitarray('11110000', 'little'), b'\x00\x0f'),
                 (bitarray('11110000', 'big'),    b'\x10\xf0'),
         ]:
-            self.assertEqual(serialize(a), s)
-            self.assertEQUAL(deserialize(s), a)
+            self.assertEqual(serialize(a), b)
+            self.assertEQUAL(deserialize(b), a)
 
     def test_zeros_and_ones(self):
         for endian in 'little', 'big':
@@ -855,24 +890,36 @@ class TestsSerialization(unittest.TestCase, Util):
     def test_wrong_args(self):
         self.assertRaises(TypeError, serialize, '0')
         self.assertRaises(TypeError, serialize, bitarray(), 1)
-        self.assertRaises(TypeError, deserialize, unicode('01'))
+        self.assertRaises(TypeError, deserialize, u'01')
         self.assertRaises(TypeError, deserialize, b'\x00', 1)
 
     def test_invalid_bytes(self):
         self.assertRaises(ValueError, deserialize, b'')
 
         for i in range(256):
-            b1 = bytes(bytearray([i]))
+            b = bytes(bytearray([i]))
             if i == 0 or i == 16:
-                self.assertEqual(deserialize(b1), bitarray())
+                self.assertEqual(deserialize(b), bitarray())
             else:
-                self.assertRaises(ValueError, deserialize, b1)
+                self.assertRaises(ValueError, deserialize, b)
 
-            b2 = bytes(bytearray([i, 0]))
+            b += b'\0'
             if i < 32 and i % 16 < 8:
-                self.assertEqual(deserialize(b2), zeros(8 - i % 8))
+                self.assertEqual(deserialize(b), zeros(8 - i % 8))
             else:
-                self.assertRaises(ValueError, deserialize, b2)
+                self.assertRaises(ValueError, deserialize, b)
+
+    def test_bits_ignored(self):
+        # the unused bits (with the last bytes) are ignored
+        for b, a in [
+                (b'\x07\x01', bitarray('1', 'little')),
+                (b'\x07\x03', bitarray('1', 'little')),
+                (b'\x07\xff', bitarray('1', 'little')),
+                (b'\x17\x80', bitarray('1', 'big')),
+                (b'\x17\xc0', bitarray('1', 'big')),
+                (b'\x17\xff', bitarray('1', 'big')),
+        ]:
+            self.assertEQUAL(deserialize(b), a)
 
     def test_random(self):
         for a in self.randombitarrays():
