@@ -9,18 +9,17 @@ from __future__ import absolute_import
 
 import os
 import sys
-import binascii
 
 from bitarray import bitarray, bits2bytes, get_default_endian
 
 from bitarray._util import (count_n, rindex,
-                            count_and, count_or, count_xor, subset,
-                            serialize, _swap_hilo_bytes, _set_bato)
+                            count_and, count_or, count_xor, subset, parity,
+                            serialize, ba2hex, _hex2ba, _set_bato)
 
 
 __all__ = ['zeros', 'urandom', 'pprint', 'make_endian', 'rindex', 'strip',
            'count_n', 'count_and', 'count_or', 'count_xor', 'subset',
-           'ba2hex', 'hex2ba', 'ba2int', 'int2ba',
+           'parity', 'ba2hex', 'hex2ba', 'ba2int', 'int2ba',
            'serialize', 'deserialize', 'huffman_code']
 
 
@@ -143,28 +142,6 @@ Allowed values for mode are the strings: `left`, `right`, `both`
     return a[first:last + 1]
 
 
-def ba2hex(a):
-    """ba2hex(bitarray, /) -> hexstr
-
-Return a string containing with hexadecimal representation of
-the bitarray (which has to be multiple of 4 in length).
-"""
-    if not isinstance(a, bitarray):
-        raise TypeError("bitarray expected, got '%s'" % type(a).__name__)
-
-    if len(a) % 4:
-        raise ValueError("bitarray length not multiple of 4")
-
-    b = a.tobytes()
-    if a.endian() == 'little':
-        b = b.translate(_swap_hilo_bytes)
-
-    s = binascii.hexlify(b)
-    if len(a) % 8:
-        s = s[:-1]
-    return s if _is_py2 else s.decode()
-
-
 def hex2ba(s, endian=None):
     """hex2ba(hexstr, /, endian=None) -> bitarray
 
@@ -174,18 +151,13 @@ hexstr may contain any number of hex digits (upper or lower case).
     if not isinstance(s, (str, unicode if _is_py2 else bytes)):
         raise TypeError("str expected, got: '%s'" % type(s).__name__)
 
-    strlen = len(s)
-    if strlen % 2:
-        s = s + ('0' if isinstance(s, str) else b'0')
+    if isinstance(s, unicode if _is_py2 else str):
+        s = s.encode('ascii')
+    assert isinstance(s, bytes)
 
-    a = bitarray(0, get_default_endian() if endian is None else endian)
-    b = binascii.unhexlify(s)
-    if a.endian() == 'little':
-        b = b.translate(_swap_hilo_bytes)
-    a.frombytes(b)
-
-    if strlen % 2:
-        del a[-4:]
+    a = bitarray(4 * len(s),
+                 get_default_endian() if endian is None else endian)
+    _hex2ba(a, s)
     return a
 
 
